#include "qpixmap.h"
#include "Fl_Image.H"
#include "fl_draw.H"

#ifndef _NANOX
#include <Imlib.h>
static ImlibData * m_id;
extern Display * fl_display;
extern Window fl_window;
#else

#include "unistd.h"
#include <nano-X.h>

extern int fl_display;
extern int fl_window;
extern GR_GC_ID fl_gc;
#endif

bool QPixmap::m_bInitialized;

bool QPixmap::loadFromData( const QByteArray & data, const char * format,
			    int conversion_flags)
{
   // This is a bit of a hack
  char namebuf[1024];
  tmpnam(namebuf);
  FILE * ptr;
  bool ret;

  ptr=fopen(namebuf,"w");

  if(ptr==NULL) {
    cerr << "Couldn't make temporary file for QPixmap::loadFromData!\n";
    return false;
  }

  unsigned int loopc;

  char * buf = data.buffer();

  for(loopc=0;loopc<data.length();loopc++) {
    fputc(buf[loopc],ptr);
  }

  fclose(ptr);

#ifndef _NANOX

  ret=load(namebuf);

  //  Delete the file
  unlink(namebuf);
#else


  m_Filename = namebuf;
  cerr << "Loaded image " << m_Filename << "\n";
  m_nPix_W = m_nWidth = 100;
  m_nPix_H = m_nHeight = 100;
  m_IM = 1;
  ret = true;
#endif

  return ret;
}

bool QPixmap::load ( const QString & fileName, 
		     const char * format, ColorMode mode )
{
#ifndef _NANOX
  if(!m_bInitialized) {
    m_id = Imlib_init(fl_display);
    if(!m_id)
      return false;
    m_bInitialized = 1;
  }

  ImlibImage * im;
  im=Imlib_load_image(m_id,(char *)fileName);
  
  m_IM = im;

  if(im==0) {
    cerr << "Errk! Failed to load QPixmap " << fileName << "\n";
    return false;
  }
  m_nPix_W = m_nWidth = im->rgb_width;
  m_nPix_H = m_nHeight = im->rgb_height;

  // Remove old image if there is one
  if(m_Pixmap) {
    Imlib_free_pixmap(m_id,(Pixmap)m_Pixmap);
  }
  Imlib_render(m_id,im,m_nPix_W,m_nPix_H);
  
  m_Pixmap = (unsigned long)Imlib_move_image(m_id,im);
  m_Mask = (unsigned long)Imlib_move_mask(m_id,im);
#else
  m_nPix_W = m_nWidth = 100;
  m_nPix_H = m_nHeight = 100;
  m_IM = 1;

#endif // !_NANOX
  return true;
}

QPixmap::~QPixmap()
{
#ifndef _NANOX
  if(m_id && m_IM)
    Imlib_destroy_image(m_id,m_IM);
#else
  //  unlink(m_Filename);

#endif
}

void QPixmap::drawPixmap(const QPoint & point, const QRect & rect)
{
  int x = (parent() ?  parent()->x() + point.x() : point.x())-1;
  int y = (parent() ?  parent()->y() + point.y() : point.y())-1;

#ifndef _NANOX
  Imlib_paste_image(m_id, m_IM, fl_window,
		    x,y,rect.width(),rect.height());
#else
  cerr << "Drawing image " << m_Filename << "\n";
  GrDrawImageFromFile(fl_window, fl_gc, x, y, rect.width(), rect.height(), 
		      GR_FALSE, m_Filename);
#endif


}
